package slimeknights.tconstruct.tools;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagList;
import net.minecraftforge.event.entity.living.LivingHurtEvent;
import net.minecraftforge.event.entity.player.PlayerEvent;
import net.minecraftforge.event.world.BlockEvent;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;

import slimeknights.tconstruct.library.TinkerRegistry;
import slimeknights.tconstruct.library.tools.ToolCore;
import slimeknights.tconstruct.library.traits.ITrait;
import slimeknights.tconstruct.library.utils.TagUtil;
import slimeknights.tconstruct.library.utils.ToolHelper;
import slimeknights.tconstruct.tools.events.TinkerToolEvent;

public class TraitEvents {

  @SubscribeEvent
  public void mineSpeed(PlayerEvent.BreakSpeed event) {
    ItemStack tool = event.entityPlayer.field_71071_by.func_70448_g();

    if(isTool(tool) && !ToolHelper.isBroken(tool)) {
      NBTTagList list = TagUtil.getTraitsTagList(tool);
      for(int i = 0; i < list.func_74745_c(); i++) {
        ITrait trait = TinkerRegistry.getTrait(list.func_150307_f(i));
        if(trait != null) {
          trait.miningSpeed(tool, event);
        }
      }
    }
  }

  @SubscribeEvent
  public void blockBreak(BlockEvent.BreakEvent event) {
    ItemStack tool = event.getPlayer().field_71071_by.func_70448_g();

    if(isTool(tool) && !ToolHelper.isBroken(tool)) {
      NBTTagList list = TagUtil.getTraitsTagList(tool);
      for(int i = 0; i < list.func_74745_c(); i++) {
        ITrait trait = TinkerRegistry.getTrait(list.func_150307_f(i));
        if(trait != null) {
          trait.beforeBlockBreak(tool, event);
        }
      }
    }
  }

  @SubscribeEvent
  public void blockDropEvent(BlockEvent.HarvestDropsEvent event) {
    if(event.harvester == null) {
      return;
    }
    ItemStack tool = event.harvester.field_71071_by.func_70448_g();

    if(isTool(tool) && !ToolHelper.isBroken(tool)) {
      NBTTagList list = TagUtil.getTraitsTagList(tool);
      for(int i = 0; i < list.func_74745_c(); i++) {
        ITrait trait = TinkerRegistry.getTrait(list.func_150307_f(i));
        if(trait != null) {
          trait.blockHarvestDrops(tool, event);
        }
      }
    }
  }

  @SubscribeEvent
  public void playerBlockEvent(LivingHurtEvent event) {
    if(event.entity == null || !(event.entity instanceof EntityPlayer) || !((EntityPlayer) event.entity).func_70632_aY()) {
      return;
    }
    // item in use has to be current item, otherwise MC stops using it
    ItemStack tool = ((EntityPlayer) event.entity).field_71071_by.func_70448_g();

    if(isTool(tool) && !ToolHelper.isBroken(tool)) {
      NBTTagList list = TagUtil.getTraitsTagList(tool);
      for(int i = 0; i < list.func_74745_c(); i++) {
        ITrait trait = TinkerRegistry.getTrait(list.func_150307_f(i));
        if(trait != null) {
          trait.onBlock(tool, (EntityPlayer) event.entity, event);
        }
      }
    }
  }

  @SubscribeEvent
  public void onRepair(TinkerToolEvent.OnRepair event) {
    ItemStack tool = event.itemStack;

    NBTTagList list = TagUtil.getTraitsTagList(tool);
    for(int i = 0; i < list.func_74745_c(); i++) {
      ITrait trait = TinkerRegistry.getTrait(list.func_150307_f(i));
      if(trait != null) {
        trait.onRepair(tool, event.amount);
      }
    }
  }

  private boolean isTool(ItemStack stack) {
    return stack != null && stack.func_77973_b() instanceof ToolCore;
  }
}
